{% load i18n suit_tags %}

<select data-name="{{ field_name }}" class="form-control search-filter{% if spec.lookup_val or spec.used_parameters %} active{% endif %}" style="max-width: 200px">
{% for choice in choices %}
  {% if forloop.first %}
    <option value="">{{ title|capfirst }}</option>
    <option value="">---</option>
  {% else %}
    <option
      {% if choice.additional %} data-additional="{{ choice.additional }}"{% endif %} data-name="{{ choice.name }}" value="{{ choice.val }}" {{ choice.selected|yesno:' selected=selected,' }}>
      {% if choice.selected %}{{ title|capfirst }}: {% endif %}{{ choice.display }}</option>
  {% endif %}
{% endfor %}
  </select>
